//
// Created by philipp on 30.11.17.
//

#include <vector>
#include <algorithm>
#include <iostream>
#include "../include/PlatoonProtocolLib/Vehicle.h"
#include "../include/PlatoonProtocolLib/Utils.h"

using namespace platoonProtocol;

constexpr std::uint16_t Vehicle::UDP_PORT;
constexpr PlatoonConfig Vehicle::DEFAULT_PLATOON_CONFIG;

Vehicle::Vehicle(networking::Networking & net,
                 const NetworkInfo & info,
                 Vehicle::Role role)
    : net(net)
      , role(role)
      , myInfo(info)
{
    receiver = networking::message::DatagramReceiver<PlatoonMessage>::create(net, UDP_PORT, PlatoonMessage::MAX_SIZE);
    sender = networking::message::DatagramSender<PlatoonMessage>::create(net);
}

bool Vehicle::isIdle()
{
    std::lock_guard<std::mutex> lock(stateMutex);
    return state == State::IDLE;
}

bool Vehicle::isPlatoonRunning()
{
    std::lock_guard<std::mutex> lock(stateMutex);
    return state == State::RUNNING_PLATOON;
}

void Vehicle::createPlatoon()
{
    auto self = shared_from_this();
    net.callLater(
        [self]
        { self->doCreatePlatoon(); });
}

void Vehicle::leavePlatoon()
{
    auto self = shared_from_this();
    net.callLater(
        [self]
        { self->doLeavePlatoon(); });
}

void Vehicle::stop()
{
    receiver->stop();

    sender->stop();
}

void Vehicle::setState(Vehicle::State newState)
{
    std::lock_guard<std::mutex> lock(stateMutex);
    state = newState;
}

}
